library angular2.src.change_detection.abstract_change_detector;

import 'package:angular2/src/facade/lang.dart' show isPresent;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper;
import './binding_propagation_config.dart' show BindingPropagationConfig;
import './interfaces.dart'
    show ChangeDetector, CHECK_ALWAYS, CHECK_ONCE, CHECKED, DETACHED;

class AbstractChangeDetector extends ChangeDetector {
  List lightDomChildren;
  List shadowDomChildren;
  ChangeDetector parent;
  String mode;
  BindingPropagationConfig bindingPropagationConfig;
  AbstractChangeDetector() : super() {
    this.lightDomChildren = [];
    this.shadowDomChildren = [];
    this.bindingPropagationConfig = new BindingPropagationConfig(this);
    this.mode = CHECK_ALWAYS;
  }
  addChild(ChangeDetector cd) {
    ListWrapper.push(this.lightDomChildren, cd);
    cd.parent = this;
  }
  removeChild(ChangeDetector cd) {
    ListWrapper.remove(this.lightDomChildren, cd);
  }
  addShadowDomChild(ChangeDetector cd) {
    ListWrapper.push(this.shadowDomChildren, cd);
    cd.parent = this;
  }
  remove() {
    this.parent.removeChild(this);
  }
  detectChanges() {
    this._detectChanges(false);
  }
  checkNoChanges() {
    this._detectChanges(true);
  }
  _detectChanges(bool throwOnChange) {
    if (identical(this.mode, DETACHED) || identical(this.mode, CHECKED)) return;
    this.detectChangesInRecords(throwOnChange);
    this._detectChangesInLightDomChildren(throwOnChange);
    this.notifyOnAllChangesDone();
    this._detectChangesInShadowDomChildren(throwOnChange);
    if (identical(this.mode, CHECK_ONCE)) this.mode = CHECKED;
  }
  detectChangesInRecords(bool throwOnChange) {}
  notifyOnAllChangesDone() {}
  _detectChangesInLightDomChildren(bool throwOnChange) {
    var c = this.lightDomChildren;
    for (var i = 0; i < c.length; ++i) {
      c[i]._detectChanges(throwOnChange);
    }
  }
  _detectChangesInShadowDomChildren(bool throwOnChange) {
    var c = this.shadowDomChildren;
    for (var i = 0; i < c.length; ++i) {
      c[i]._detectChanges(throwOnChange);
    }
  }
  markPathToRootAsCheckOnce() {
    var c = this;
    while (isPresent(c) && c.mode != DETACHED) {
      if (identical(c.mode, CHECKED)) c.mode = CHECK_ONCE;
      c = c.parent;
    }
  }
}
